<?php

namespace B2buy\Shipping\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use B2buy\Shipping\Http\Requests\RegionAreaRequest;
use B2buy\Shipping\Interfaces\RegionAreaRepositoryInterface;
use B2buy\Shipping\Models\RegionArea;

/**
 * Resource controller class for region_area.
 */
class RegionAreaResourceController extends BaseController
{

    /**
     * Initialize region_area resource controller.
     *
     * @param type RegionAreaRepositoryInterface $region_area
     *
     * @return null
     */
    public function __construct(RegionAreaRepositoryInterface $region_area)
    {
        parent::__construct();
        $this->repository = $region_area;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\B2buy\Shipping\Repositories\Criteria\RegionAreaResourceCriteria::class);
    }

    /**
     * Display a list of region_area.
     *
     * @return Response
     */
    public function index(RegionAreaRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\B2buy\Shipping\Repositories\Presenter\RegionAreaPresenter::class)
                ->$function();
        }

        $region_areas = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('shipping::region_area.names'))
            ->view('shipping::region_area.index', true)
            ->data(compact('region_areas', 'view'))
            ->output();
    }

    /**
     * Display region_area.
     *
     * @param Request $request
     * @param Model   $region_area
     *
     * @return Response
     */
    public function show(RegionAreaRequest $request, RegionArea $region_area)
    {

        if ($region_area->exists) {
            $view = 'shipping::region_area.show';
        } else {
            $view = 'shipping::region_area.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('shipping::region_area.name'))
            ->data(compact('region_area'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new region_area.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(RegionAreaRequest $request)
    {

        $region_area = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('shipping::region_area.name')) 
            ->view('shipping::region_area.create', true) 
            ->data(compact('region_area'))
            ->output();
    }

    /**
     * Create new region_area.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(RegionAreaRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $region_area                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('shipping::region_area.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('shipping/region_area/' . $region_area->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/shipping/region_area'))
                ->redirect();
        }

    }

    /**
     * Show region_area for editing.
     *
     * @param Request $request
     * @param Model   $region_area
     *
     * @return Response
     */
    public function edit(RegionAreaRequest $request, RegionArea $region_area)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('shipping::region_area.name'))
            ->view('shipping::region_area.edit', true)
            ->data(compact('region_area'))
            ->output();
    }

    /**
     * Update the region_area.
     *
     * @param Request $request
     * @param Model   $region_area
     *
     * @return Response
     */
    public function update(RegionAreaRequest $request, RegionArea $region_area)
    {
        try {
            $attributes = $request->all();

            $region_area->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('shipping::region_area.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('shipping/region_area/' . $region_area->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('shipping/region_area/' . $region_area->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the region_area.
     *
     * @param Model   $region_area
     *
     * @return Response
     */
    public function destroy(RegionAreaRequest $request, RegionArea $region_area)
    {
        try {

            $region_area->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('shipping::region_area.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('shipping/region_area/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('shipping/region_area/' . $region_area->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple region_area.
     *
     * @param Model   $region_area
     *
     * @return Response
     */
    public function delete(RegionAreaRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('shipping::region_area.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('shipping/region_area'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/shipping/region_area'))
                ->redirect();
        }

    }

    /**
     * Restore deleted region_areas.
     *
     * @param Model   $region_area
     *
     * @return Response
     */
    public function restore(RegionAreaRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('shipping::region_area.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/shipping/region_area'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/shipping/region_area/'))
                ->redirect();
        }

    }

}
